@@ -1,5 +1,6 @@
 """Defines the base dialect class."""
 
+import sys
 from typing import Set, Union, Type
 
 from sqlfluff.core.parser import (
@@ -9,11 +10,11 @@ from sqlfluff.core.parser import (
     StringParser,
 )
 from sqlfluff.core.parser.grammar.base import BaseGrammar
-from sqlfluff.core.parser.parsers import BaseParser
+from sqlfluff.core.parser.matchable import Matchable
 
-DialectElementType = Union[Type[BaseSegment], BaseGrammar, BaseParser, SegmentGenerator]
+DialectElementType = Union[Type[BaseSegment], Matchable, SegmentGenerator]
 # NOTE: Post expansion, no generators remain
-ExpandedDialectElementType = Union[Type[BaseSegment], BaseParser, BaseGrammar]
+ExpandedDialectElementType = Union[Type[BaseSegment], Matchable]
 
 
 class Dialect:
@@ -100,6 +101,14 @@ class Dialect:
             self._sets[label] = set()
         return self._sets[label]
 
+    def update_keywords_set_from_multiline_string(
+        self, set_label: str, values: str
+    ) -> None:
+        """Special function to update a keywords set from a multi-line string."""
+        self.sets(set_label).update(
+            [n.strip().upper() for n in values.strip().split("\n")]
+        )
+
     def copy_as(self, name):
         """Copy this dialect and create a new one with a different name.
 
@@ -154,6 +163,12 @@ class Dialect:
             cls = kwargs[n]
             if self._library[n] is cls:
                 continue
+            elif self._library[n] == cls:
+                # Check for replacement with a new but identical class.
+                # This would be a sign of redundant definitions in the dialect.
+                raise ValueError(
+                    f"Attempted unnecessary identical redefinition of {n!r} in {self!r}"
+                )  # pragma: no cover
 
             # To replace a segment, the replacement must either be a
             # subclass of the original, *or* it must have the same
@@ -261,19 +276,33 @@ class Dialect:
                         name, self.name
                     )
                 )
-        else:  # pragma: no cover
-            if name.endswith("KeywordSegment"):
-                keyword_tip = (
-                    " Perhaps specify the keyword? "
-                    "https://github.com/sqlfluff/sqlfluff/wiki/Contributing-Dialect-Changes#keywords"  # noqa E501
+        elif name.endswith("KeywordSegment"):  # pragma: no cover
+            keyword = name[0:-14]
+            keyword_tip = (
+                "\n\nThe syntax in the query is not (yet?) supported. Try to"
+                " narrow down your query to a minimal, reproducible case and"
+                " raise an issue on GitHub.\n\n"
+                "Or, even better, see this guide on how to help contribute"
+                " keyword and/or dialect updates:\n"
+                "https://github.com/sqlfluff/sqlfluff/wiki/Contributing-Dialect-Changes#keywords"  # noqa E501
+            )
+            # Keyword errors are common so avoid printing the whole, scary,
+            # traceback as not that useful and confusing to people.
+            sys.tracebacklimit = 0
+            raise RuntimeError(
+                (
+                    "Grammar refers to the "
+                    "{!r} keyword which was not found in the {} dialect.{}".format(
+                        keyword, self.name, keyword_tip
+                    )
                 )
-            else:
-                keyword_tip = ""
+            )
+        else:  # pragma: no cover
             raise RuntimeError(
                 (
                     "Grammar refers to "
-                    "{!r} which was not found in the {} dialect.{}".format(
-                        name, self.name, keyword_tip
+                    "{!r} which was not found in the {} dialect.".format(
+                        name, self.name
                     )
                 )
             )
@@ -337,7 +366,7 @@ class Dialect:
 
         if not found:  # pragma: no cover
             raise ValueError(
-                "Lexer struct insert before '%s' failed because tag never found."
+                f"Lexer struct insert before '{before}' failed because tag never found."
             )
         # Overwrite with the buffer once we're done
         self.lexer_matchers = buff
