@@ -141,7 +141,7 @@ class Linter:
         with open(fname, encoding=encoding, errors="backslashreplace") as target_file:
             raw_file = target_file.read()
         # Scan the raw file for config commands.
-        file_config.process_raw_file_for_config(raw_file)
+        file_config.process_raw_file_for_config(raw_file, fname)
         # Return the raw file and config
         return raw_file, file_config, encoding
 
@@ -223,6 +223,7 @@ class Linter:
         config: FluffConfig,
         recurse: bool = True,
         fname: Optional[str] = None,
+        parse_statistics: bool = False,
     ) -> Tuple[Optional[BaseSegment], List[SQLParseError]]:
         parser = Parser(config=config)
         violations = []
@@ -232,6 +233,7 @@ class Linter:
                 tokens,
                 recurse=recurse,
                 fname=fname,
+                parse_statistics=parse_statistics,
             )
         except SQLParseError as err:
             linter_logger.info("PARSING FAILED! : %s", err)
@@ -381,6 +383,7 @@ class Linter:
         cls,
         rendered: RenderedFile,
         recurse: bool = True,
+        parse_statistics: bool = False,
     ) -> ParsedString:
         """Parse a rendered file."""
         t0 = time.monotonic()
@@ -403,6 +406,7 @@ class Linter:
                 rendered.config,
                 recurse=recurse,
                 fname=rendered.fname,
+                parse_statistics=parse_statistics,
             )
             violations += pvs
         else:
@@ -885,6 +889,7 @@ class Linter:
         recurse: bool = True,
         config: Optional[FluffConfig] = None,
         encoding: str = "utf-8",
+        parse_statistics: bool = False,
     ) -> ParsedString:
         """Parse a string."""
         violations: List[SQLBaseError] = []
@@ -897,7 +902,7 @@ class Linter:
         config = config or self.config
 
         # Scan the raw file for config commands.
-        config.process_raw_file_for_config(in_str)
+        config.process_raw_file_for_config(in_str, fname)
         rendered = self.render_string(in_str, fname, config, encoding)
         violations += rendered.templater_violations
 
@@ -905,7 +910,9 @@ class Linter:
         if self.formatter:
             self.formatter.dispatch_parse_header(fname)
 
-        return self.parse_rendered(rendered, recurse=recurse)
+        return self.parse_rendered(
+            rendered, recurse=recurse, parse_statistics=parse_statistics
+        )
 
     def fix(
         self,
@@ -1238,6 +1245,7 @@ class Linter:
         self,
         path: str,
         recurse: bool = True,
+        parse_statistics: bool = False,
     ) -> Iterator[ParsedString]:
         """Parse a path of sql files.
 
@@ -1261,4 +1269,5 @@ class Linter:
                 recurse=recurse,
                 config=config,
                 encoding=encoding,
+                parse_statistics=parse_statistics,
             )
