@@ -13,6 +13,7 @@ from sqlfluff.core.parser.segments import (
     SymbolSegment,
     Indent,
     Dedent,
+    ImplicitIndent,
     SegmentGenerator,
     IdentitySet,
 )
@@ -33,7 +34,7 @@ from sqlfluff.core.parser.grammar import (
 )
 from sqlfluff.core.parser.parsers import (
     StringParser,
-    NamedParser,
+    TypedParser,
     RegexParser,
     MultiStringParser,
 )
@@ -55,6 +56,7 @@ __all__ = (
     "SymbolSegment",
     "Indent",
     "Dedent",
+    "ImplicitIndent",
     "SegmentGenerator",
     "Sequence",
     "GreedyUntil",
@@ -71,7 +73,7 @@ __all__ = (
     "Conditional",
     "StringParser",
     "MultiStringParser",
-    "NamedParser",
+    "TypedParser",
     "RegexParser",
     "PositionMarker",
     "Lexer",
