@@ -228,7 +228,6 @@ class BaseGrammar(Matchable):
         matchers: List[MatchableType],
         parse_context: ParseContext,
         trim_noncode=True,
-        terminators: Optional[List[MatchableType]] = None,
     ) -> Tuple[MatchResult, Optional[MatchableType]]:
         """Return longest match from a selection of matchers.
 
@@ -239,9 +238,21 @@ class BaseGrammar(Matchable):
         Returns:
             `tuple` of (match_object, matcher).
 
+        NOTE: This matching method is the workhorse of the parser. It's performance
+        can be monitored using the `parse_stats` object on the context.
         """
         terminated = False
 
+        parse_context.increment("ltm_calls")
+        # NOTE: The use of terminators is only available via the context.
+        # They are set in that way to allow appropriate inheritance rather
+        # than only being used in a per-grammar basis.
+        if parse_context.terminators:
+            parse_context.increment("ltm_calls_w_ctx_terms")
+            terminators = parse_context.terminators
+        else:
+            terminators = []
+
         # Have we been passed an empty list?
         if len(segments) == 0:  # pragma: no cover
             return MatchResult.from_empty(), None
@@ -264,7 +275,7 @@ class BaseGrammar(Matchable):
 
         best_match_length = 0
         # iterate at this position across all the matchers
-        for matcher in matchers:
+        for idx, matcher in enumerate(matchers):
             # Check parse cache.
             matcher_key = matcher.cache_key()
             res_match: Optional[MatchResult] = parse_context.check_parse_cache(
@@ -290,6 +301,7 @@ class BaseGrammar(Matchable):
 
             if res_match.is_complete():
                 # Just return it! (WITH THE RIGHT OTHER STUFF)
+                parse_context.increment("complete_match")
                 if trim_noncode:
                     return (
                         MatchResult.from_matched(
@@ -309,7 +321,12 @@ class BaseGrammar(Matchable):
                     # end earlier, and claim an effectively "complete" match.
                     # NOTE: This means that by specifying terminators, we can
                     # significantly increase performance.
-                    if terminators:
+                    if idx == len(matchers) - 1:
+                        # If it's the last option - no need to check terminators.
+                        # We're going to end anyway, so we can skip that step.
+                        terminated = True
+                        break
+                    elif terminators:
                         _, segs, _ = trim_non_code_segments(
                             best_match[0].unmatched_segments
                         )
@@ -330,9 +347,29 @@ class BaseGrammar(Matchable):
             # Eventually there might be a performance gain from doing that sensibly
             # here.
 
+        if terminated:
+            parse_context.increment("terminated_match")
+        else:
+            parse_context.increment("unterminated_match")
+
         # If we get here, then there wasn't a complete match. If we
         # has a best_match, return that.
         if best_match_length > 0:
+            # If not terminated, keep track of what the next token would
+            # have been if we had been able to terminate using it.
+            if not terminated:
+                if best_match[0].unmatched_segments:
+                    for seg in best_match[0].unmatched_segments:
+                        if seg.is_code:
+                            break
+                    next_seg = seg.raw_segments[0].raw_upper
+                else:  # pragma: no cover
+                    # NOTE: I don't think this clause should ever
+                    # occur, but it's included so that if it does happen
+                    # we don't get an exception and can better debug.
+                    next_seg = "<NONE>"
+                parse_context.parse_stats["next_counts"][next_seg] += 1
+
             if trim_noncode:
                 return (
                     MatchResult(
@@ -800,6 +837,14 @@ class Ref(BaseGrammar):
     def __init__(self, *args: str, **kwargs):
         # Any patterns to _prevent_ a match.
         self.exclude = kwargs.pop("exclude", None)
+        # The intent here is that if we match something, and then the _next_
+        # item is one of these, we can safely conclude it's a "total" match.
+        # In those cases, we return early without considering more options.
+        # Terminators don't take effect directly within this grammar, but
+        # the Ref grammar is an effective place to manage the terminators
+        # inherited via the context.
+        self.terminators = kwargs.pop("terminators", None)
+        self.reset_terminators = kwargs.pop("reset_terminators", False)
         super().__init__(*args, **kwargs)
 
     @cached_method_for_parse_context
@@ -869,6 +914,12 @@ class Ref(BaseGrammar):
         # which would prevent the rest of this grammar from matching.
         if self.exclude:
             with parse_context.deeper_match() as ctx:
+                # NOTE: Not covered because `exclude` and `teminators` aren't
+                # currently used together in any dialect.
+                if self.reset_terminators:  # pragma: no cover
+                    ctx.clear_terminators()
+                if self.terminators:  # pragma: no cover
+                    ctx.push_terminators(self.terminators)
                 if self.exclude.match(segments, parse_context=ctx):
                     return MatchResult.from_unmatched(segments)
 
@@ -893,6 +944,10 @@ class Ref(BaseGrammar):
         # Match against that. NB We're not incrementing the match_depth here.
         # References shouldn't really count as a depth of match.
         with parse_context.matching_segment(self._get_ref()) as ctx:
+            if self.reset_terminators:
+                ctx.clear_terminators()
+            if self.terminators:
+                ctx.push_terminators(self.terminators)
             resp = elem.match(segments=segments, parse_context=ctx)
         if not resp:
             parse_context.denylist.mark(self_name, seg_tuple)
