@@ -151,19 +151,17 @@ class Delimited(OneOf):
                         break
 
                 with parse_context.deeper_match() as ctx:
+                    if delimiter_matchers and elements != delimiter_matchers:
+                        ctx.push_terminators(delimiter_matchers)
                     match, _ = self._longest_trimmed_match(
                         segments=seg_content,
                         matchers=elements,
                         parse_context=ctx,
                         # We've already trimmed
                         trim_noncode=False,
-                        terminators=delimiter_matchers
-                        if elements != delimiter_matchers
-                        else None,
                     )
 
                 if match:
-
                     if elements == delimiter_matchers:
                         delimiters += 1
                         matched_delimiter = True
@@ -178,7 +176,6 @@ class Delimited(OneOf):
                     unmatched_segments = match.unmatched_segments
 
                     if match.is_complete():
-
                         matched_segments += (
                             pre_non_code + match.matched_segments + post_non_code
                         )
@@ -208,14 +205,12 @@ class Delimited(OneOf):
                 return MatchResult.from_unmatched(matched_segments + unmatched_segments)
 
         if matched_delimiter and not self.allow_trailing:
-
             if not unmatched_segments:
                 return MatchResult.from_unmatched(matched_segments + unmatched_segments)
             else:
                 return MatchResult(cached_matched_segments, cached_unmatched_segments)
 
         if not has_matched_segs:
-
             return MatchResult.from_unmatched(matched_segments + unmatched_segments)
 
         if not unmatched_segments:
