@@ -232,6 +232,12 @@ class Sequence(BaseGrammar):
         return MatchResult(
             BaseSegment._position_segments(
                 matched_segments.matched_segments + meta_pre_nc + meta_post_nc,
+                # Repositioning only meta segments at this stage does increase the
+                # risk of leakage a little (by not fully copying everything on
+                # return), but it does drastically improve performance. Future
+                # work may involve more immutable segments or a smarter way
+                # of isolating them.
+                metas_only=True,
             ),
             unmatched_segments,
         )
@@ -349,14 +355,18 @@ class Bracketed(Sequence):
                 return MatchResult.from_unmatched(segments)
 
             # Look for the closing bracket
-            content_segs, end_match, _ = self._bracket_sensitive_look_ahead_match(
-                segments=seg_buff,
-                matchers=[end_bracket],
-                parse_context=parse_context,
-                start_bracket=start_bracket,
-                end_bracket=end_bracket,
-                bracket_pairs_set=self.bracket_pairs_set,
-            )
+            with parse_context.deeper_match() as ctx:
+                # Within the brackets, clear any inherited terminators.
+                ctx.clear_terminators()
+                content_segs, end_match, _ = self._bracket_sensitive_look_ahead_match(
+                    segments=seg_buff,
+                    matchers=[end_bracket],
+                    parse_context=ctx,
+                    start_bracket=start_bracket,
+                    end_bracket=end_bracket,
+                    bracket_pairs_set=self.bracket_pairs_set,
+                )
+
             if not end_match:  # pragma: no cover
                 raise SQLParseError(
                     "Couldn't find closing bracket for opening bracket.",
@@ -400,6 +410,8 @@ class Bracketed(Sequence):
         # Match the content using super. Sequence will interpret the content of the
         # elements.
         with parse_context.deeper_match() as ctx:
+            # Within the brackets, clear any inherited terminators.
+            ctx.clear_terminators()
             content_match = super().match(content_segs, parse_context=ctx)
 
         # We require a complete match for the content (hopefully for obvious reasons)
