@@ -1,21 +1,30 @@
 """Base implementation for the plugin."""
 
 import os.path
+
+from typing import List, Type
+
 from sqlfluff.core.config import ConfigLoader
 from sqlfluff.core.plugin import hookimpl
 from sqlfluff.core.rules.config_info import STANDARD_CONFIG_INFO_DICT
 from sqlfluff.core.rules.loader import get_rules_from_path
-from sqlfluff.core.templaters import core_templaters
+from sqlfluff.core.rules import BaseRule
+from sqlfluff.core.templaters import core_templaters, RawTemplater
 
 
 @hookimpl
-def get_rules():
-    """Get plugin rules."""
+def get_rules() -> List[Type[BaseRule]]:
+    """Get plugin rules.
+
+    NOTE: All standard rules will eventually be loaded as
+    plugins and so before 2.0.0, once all legacy plugin definitions
+    are migrated, this function will be amended to return no rules.
+    """
     return get_rules_from_path()
 
 
 @hookimpl
-def get_templaters():
+def get_templaters() -> List[Type[RawTemplater]]:
     """Get templaters."""
     return core_templaters()
 
