@@ -20,6 +20,7 @@ UNRESERVED_KEYWORDS = [
     "ASOF",
     "AST",
     "ASYNC",
+    "ATOMIC",
     "ATTACH",
     "BETWEEN",
     "BOTH",
@@ -93,6 +94,7 @@ UNRESERVED_KEYWORDS = [
     "KILL",
     "LAST",
     "LAYOUT",
+    "LAZY",
     "LEADING",
     "LIFETIME",
     "LIKE",
@@ -102,6 +104,8 @@ UNRESERVED_KEYWORDS = [
     "LOGS",
     "MATERIALIZE",
     "MATERIALIZED",
+    "MATERIALIZEDMYSQL",
+    "MATERIALIZEDPOSTGRESQL",
     "MAX",
     "MERGES",
     "MIN",
@@ -113,16 +117,19 @@ UNRESERVED_KEYWORDS = [
     "MOVE",
     "MOVES",
     "MUTATION",
+    "MYSQL",
     "NAN_SQL",
     "NO",
     "NOT",
-    "NULL_SQL",
     "NULLS",
+    "NULL_SQL",
     "OFFSET",
     "OPTIMIZE",
     "OR",
     "OUTFILE",
+    "OVERRIDE",
     "POPULATE",
+    "POSTGRESQL",
     "PREWHERE",
     "PRIMARY",
     "PROFILE",
@@ -147,6 +154,7 @@ UNRESERVED_KEYWORDS = [
     "SETTINGS",
     "SHOW",
     "SOURCE",
+    "SQLITE",
     "START",
     "STOP",
     "SUBSTRING",
