@@ -1306,12 +1306,18 @@ class AlterTableStatementSegment(BaseSegment):
                         ),
                         # Rename index
                         Sequence(
-                            "RENAME",
                             OneOf("INDEX", "KEY"),
                             Ref("IndexReferenceSegment"),
                             "TO",
                             Ref("IndexReferenceSegment"),
                         ),
+                        # Rename column
+                        Sequence(
+                            "COLUMN",
+                            Ref("ColumnReferenceSegment"),
+                            "TO",
+                            Ref("ColumnReferenceSegment"),
+                        ),
                     ),
                 ),
                 # Enable/Disable updating nonunique indexes
