@@ -35,18 +35,28 @@ redshift_dialect = postgres_dialect.copy_as("redshift")
 
 # Set Keywords
 redshift_dialect.sets("unreserved_keywords").clear()
-redshift_dialect.sets("unreserved_keywords").update(
-    [n.strip().upper() for n in redshift_unreserved_keywords.split("\n")]
+redshift_dialect.update_keywords_set_from_multiline_string(
+    "unreserved_keywords", redshift_unreserved_keywords
 )
 
 redshift_dialect.sets("reserved_keywords").clear()
-redshift_dialect.sets("reserved_keywords").update(
-    [n.strip().upper() for n in redshift_reserved_keywords.split("\n")]
+redshift_dialect.update_keywords_set_from_multiline_string(
+    "reserved_keywords", redshift_reserved_keywords
 )
 
 redshift_dialect.sets("bare_functions").clear()
 redshift_dialect.sets("bare_functions").update(
-    ["current_date", "sysdate", "current_timestamp"]
+    [
+        "current_date",
+        "sysdate",
+        "current_time",
+        "current_timestamp",
+        "user",
+        "current_user",
+        "current_aws_account",
+        "current_namespace",
+        "current_user_id",
+    ]
 )
 
 redshift_dialect.sets("date_part_function_name").update(
@@ -57,7 +67,7 @@ redshift_dialect.sets("date_part_function_name").update(
 # https://docs.aws.amazon.com/redshift/latest/dg/r_Dateparts_for_datetime_functions.html
 redshift_dialect.sets("datetime_units").update(
     [
-        # millenium
+        # millennium
         "MILLENNIUM",
         "MILLENNIA",
         "MIL",
@@ -177,33 +187,17 @@ redshift_dialect.replace(
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
     ),
-    ColumnReferenceSegment=Delimited(
-        Sequence(
-            ansi.ColumnReferenceSegment,
-            AnyNumberOf(Ref("ArrayAccessorSegment")),
-        ),
-        delimiter=OneOf(
-            Ref("DotSegment"), Sequence(Ref("DotSegment"), Ref("DotSegment"))
-        ),
-        terminator=OneOf(
-            "ON",
-            "AS",
-            "USING",
-            Ref("CommaSegment"),
-            Ref("CastOperatorSegment"),
-            Ref("BinaryOperatorGrammar"),
-            Ref("ColonSegment"),
-            Ref("DelimiterGrammar"),
-            Ref("JoinLikeClauseGrammar"),
-        ),
-        allow_gaps=False,
-    ),
 )
 
 redshift_dialect.patch_lexer_matchers(
     [
         # add optional leading # to code for temporary tables
-        RegexLexer("code", r"#?[0-9a-zA-Z_]+[0-9a-zA-Z_$]*", CodeSegment),
+        RegexLexer(
+            "code",
+            r"#?[0-9a-zA-Z_]+[0-9a-zA-Z_$]*",
+            CodeSegment,
+            segment_kwargs={"type": "code"},
+        ),
     ]
 )
 
@@ -343,6 +337,26 @@ class DateTimeTypeIdentifier(BaseSegment):
     )
 
 
+class BracketedArguments(ansi.BracketedArguments):
+    """A series of bracketed arguments.
+
+    e.g. the bracketed part of numeric(1, 3)
+    """
+
+    match_grammar = Bracketed(
+        # The brackets might be empty for some cases...
+        Delimited(
+            OneOf(
+                Ref("LiteralGrammar"),
+                # In redshift, character types offer on optional MAX
+                # keyword in their parameters.
+                "MAX",
+            ),
+            optional=True,
+        ),
+    )
+
+
 class DatatypeSegment(BaseSegment):
     """A data type segment.
 
@@ -369,10 +383,7 @@ class DatatypeSegment(BaseSegment):
         # numeric types [precision ["," scale])]
         Sequence(
             OneOf("DECIMAL", "NUMERIC"),
-            Bracketed(
-                Delimited(Ref("NumericLiteralSegment")),
-                optional=True,
-            ),
+            Ref("BracketedArguments", optional=True),
         ),
         # character types
         OneOf(
@@ -385,13 +396,7 @@ class DatatypeSegment(BaseSegment):
                     Sequence("CHARACTER", "VARYING"),
                     "NVARCHAR",
                 ),
-                Bracketed(
-                    OneOf(
-                        Ref("NumericLiteralSegment"),
-                        "MAX",
-                    ),
-                    optional=True,
-                ),
+                Ref("BracketedArguments", optional=True),
             ),
             "BPCHAR",
             "TEXT",
@@ -415,10 +420,7 @@ class DatatypeSegment(BaseSegment):
                 "VARBINARY",
                 Sequence("BINARY", "VARYING"),
             ),
-            Bracketed(
-                Ref("NumericLiteralSegment"),
-                optional=True,
-            ),
+            Ref("BracketedArguments", optional=True),
         ),
         "ANYELEMENT",
     )
@@ -680,7 +682,7 @@ class AlterTableActionSegment(BaseSegment):
             Ref("ColumnReferenceSegment"),
             Ref("DatatypeSegment"),
             Sequence("DEFAULT", Ref("ExpressionSegment"), optional=True),
-            Sequence("COLLATE", Ref("QuotedLiteralSegment"), optional=True),
+            Sequence("COLLATE", Ref("CollationReferenceSegment"), optional=True),
             AnyNumberOf(Ref("ColumnConstraintSegment")),
         ),
         Sequence(
@@ -1181,11 +1183,7 @@ class CreateLibraryStatementSegment(BaseSegment):
 
     match_grammar = Sequence(
         "CREATE",
-        Sequence(
-            "OR",
-            "REPLACE",
-            optional=True,
-        ),
+        Ref("OrReplaceGrammar", optional=True),
         "LIBRARY",
         Ref("ObjectReferenceSegment"),
         "LANGUAGE",
@@ -1497,7 +1495,7 @@ class CreateSchemaStatementSegment(BaseSegment):
     """A `CREATE SCHEMA` statement.
 
     https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_SCHEMA.html
-    TODO: support optional SCHEMA_ELEMENT
+    TODO: support optional SCHEMA_ELEMENT (should mostly be provided by ansi)
     """
 
     type = "create_schema_statement"
@@ -1510,13 +1508,13 @@ class CreateSchemaStatementSegment(BaseSegment):
                 Ref("SchemaReferenceSegment"),
                 Sequence(
                     "AUTHORIZATION",
-                    Ref("ObjectReferenceSegment"),
+                    Ref("RoleReferenceSegment"),
                     optional=True,
                 ),
             ),
             Sequence(
                 "AUTHORIZATION",
-                Ref("ObjectReferenceSegment"),
+                Ref("RoleReferenceSegment"),
             ),
         ),
         Ref("QuotaGrammar", optional=True),
@@ -1564,7 +1562,7 @@ class CreateProcedureStatementSegment(BaseSegment):
 
     match_grammar = Sequence(
         "CREATE",
-        Sequence("OR", "REPLACE", optional=True),
+        Ref("OrReplaceGrammar", optional=True),
         "PROCEDURE",
         Ref("FunctionNameSegment"),
         Ref("ProcedureParameterListSegment"),
@@ -1837,6 +1835,95 @@ class ShowDatasharesStatementSegment(BaseSegment):
     )
 
 
+class CreateRlsPolicyStatementSegment(BaseSegment):
+    """A `CREATE RLS POLICY` statement.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_RLS_POLICY.html
+    """
+
+    type = "create_rls_policy_statement"
+    match_grammar = Sequence(
+        "CREATE",
+        "RLS",
+        "POLICY",
+        Ref("ObjectReferenceSegment"),
+        Sequence(
+            "WITH",
+            Bracketed(
+                Delimited(
+                    Sequence(
+                        Ref("ColumnReferenceSegment"),
+                        Ref("DatatypeSegment"),
+                    ),
+                ),
+            ),
+            Sequence(
+                Ref.keyword("AS", optional=True),
+                Ref("AliasExpressionSegment"),
+                optional=True,
+            ),
+            optional=True,
+        ),
+        Sequence(
+            "USING",
+            Bracketed(Ref("ExpressionSegment")),
+        ),
+    )
+
+
+class ManageRlsPolicyStatementSegment(BaseSegment):
+    """An `ATTACH/DETACH RLS POLICY` statement.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_ATTACH_RLS_POLICY.html
+    https://docs.aws.amazon.com/redshift/latest/dg/r_DETACH_RLS_POLICY.html
+    """
+
+    # 1 statement for both ATTACH and DETACH since same syntax
+    type = "manage_rls_policy_statement"
+    match_grammar = Sequence(
+        OneOf("ATTACH", "DETACH"),
+        "RLS",
+        "POLICY",
+        Ref("ObjectReferenceSegment"),
+        "ON",
+        Ref.keyword("TABLE", optional=True),
+        Delimited(
+            Ref("TableReferenceSegment"),
+        ),
+        OneOf("TO", "FROM"),
+        Delimited(
+            OneOf(
+                Sequence(
+                    Ref.keyword("ROLE", optional=True),
+                    Ref("RoleReferenceSegment"),
+                ),
+                "PUBLIC",
+            ),
+        ),
+    )
+
+
+class DropRlsPolicyStatementSegment(BaseSegment):
+    """A `DROP RLS POLICY` statement.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_DROP_RLS_POLICY.html
+    """
+
+    type = "drop_rls_policy_statement"
+    match_grammar = Sequence(
+        "DROP",
+        "RLS",
+        "POLICY",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("ObjectReferenceSegment"),
+        OneOf(
+            "CASCADE",
+            "RESTRICT",
+            optional=True,
+        ),
+    )
+
+
 class AnalyzeCompressionStatementSegment(BaseSegment):
     """An `ANALYZE COMPRESSION` statement.
 
@@ -1865,13 +1952,12 @@ class AnalyzeCompressionStatementSegment(BaseSegment):
     )
 
 
-class VacuumStatementSegment(BaseSegment):
+class VacuumStatementSegment(postgres.VacuumStatementSegment):
     """A `VACUUM` statement.
 
     https://docs.aws.amazon.com/redshift/latest/dg/r_VACUUM_command.html
     """
 
-    type = "vacuum_statement"
     match_grammar = Sequence(
         "VACUUM",
         OneOf(
@@ -1927,9 +2013,12 @@ class StatementSegment(postgres.StatementSegment):
             Ref("FetchStatementSegment"),
             Ref("CloseStatementSegment"),
             Ref("AnalyzeCompressionStatementSegment"),
-            Ref("VacuumStatementSegment"),
             Ref("AlterProcedureStatementSegment"),
             Ref("CallStatementSegment"),
+            Ref("CreateRlsPolicyStatementSegment"),
+            Ref("ManageRlsPolicyStatementSegment"),
+            Ref("DropRlsPolicyStatementSegment"),
+            Ref("CreateExternalFunctionStatementSegment"),
         ],
     )
 
@@ -1964,7 +2053,7 @@ class RowFormatDelimitedSegment(BaseSegment):
     https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_EXTERNAL_TABLE.html
     """
 
-    type = "row_format_deimited_segment"
+    type = "row_format_delimited_segment"
 
     match_grammar = AnySetOf(
         Sequence(
@@ -2373,12 +2462,25 @@ class FunctionSegment(ansi.FunctionSegment):
         ),
         Sequence(
             Sequence(
-                Ref(
-                    "FunctionNameSegment",
-                    exclude=OneOf(
-                        Ref("DatePartFunctionNameSegment"),
-                        Ref("ValuesClauseSegment"),
-                        Ref("ConvertFunctionNameSegment"),
+                OneOf(
+                    Ref(
+                        "FunctionNameSegment",
+                        exclude=OneOf(
+                            Ref("DatePartFunctionNameSegment"),
+                            Ref("ValuesClauseSegment"),
+                            Ref("ConvertFunctionNameSegment"),
+                        ),
+                    ),
+                    Sequence(
+                        Ref.keyword("APPROXIMATE"),
+                        Ref(
+                            "FunctionNameSegment",
+                            exclude=OneOf(
+                                Ref("DatePartFunctionNameSegment"),
+                                Ref("ValuesClauseSegment"),
+                                Ref("ConvertFunctionNameSegment"),
+                            ),
+                        ),
                     ),
                 ),
                 Bracketed(
@@ -2401,3 +2503,69 @@ class FunctionSegment(ansi.FunctionSegment):
             ),
         ),
     )
+
+
+class FromClauseSegment(ansi.FromClauseSegment):
+    """Slightly modified version which allows for using brackets for content of FROM."""
+
+    match_grammar = Sequence(
+        "FROM",
+        Delimited(
+            OptionallyBracketed(Ref("FromExpressionSegment")),
+        ),
+    )
+
+
+class CreateViewStatementSegment(BaseSegment):
+    """A `CREATE VIEW` statement."""
+
+    type = "create_view_statement"
+    # https://crate.io/docs/sql-99/en/latest/chapters/18.html#create-view-statement
+    # https://dev.mysql.com/doc/refman/8.0/en/create-view.html
+    # https://www.postgresql.org/docs/12/sql-createview.html
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "VIEW",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        # Optional list of column names
+        Ref("BracketedColumnReferenceListGrammar", optional=True),
+        "AS",
+        OptionallyBracketed(Ref("SelectableGrammar")),
+        Ref("WithNoSchemaBindingClauseSegment", optional=True),
+    )
+
+
+class CreateExternalFunctionStatementSegment(BaseSegment):
+    """A `CREATE EXTERNAL FUNCTION` segment.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/r_CREATE_EXTERNAL_FUNCTION.html
+    """
+
+    type = "create_external_function_statement"
+    match_grammar = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "EXTERNAL",
+        "FUNCTION",
+        Ref("FunctionNameSegment"),
+        Bracketed(
+            Delimited(
+                Ref("DatatypeSegment"),
+                optional=True,
+            ),
+        ),
+        "RETURNS",
+        Ref("DatatypeSegment"),
+        OneOf("VOLATILE", "STABLE", "IMMUTABLE"),
+        OneOf("LAMBDA", "SAGEMAKER"),
+        Ref("QuotedLiteralSegment"),
+        "IAM_ROLE",
+        OneOf("DEFAULT", Ref("QuotedLiteralSegment")),
+        Sequence(
+            "RETRY_TIMEOUT",
+            Ref("NumericLiteralSegment"),
+            optional=True,
+        ),
+    )
